! Ladder Game Clone
! Nils M Holm, 2023
! In the public domain / 0BSD license

use t3x: t;
use console: con;

const	MAXBARREL = 20;
const	MAXDROP = 6;
const	MAXLEVEL = 7;

struct	PARMS = P_DUMMY, P_BMAX, P_BCAD;

var	VT52;
var	Debug;

var	Scores::128;

var	Levels;
var	Screen[24], Buffer::24*80;
var	Register;
var	Jumpd, Jumpp;
var	Climbd;

var	Bx[MAXBARREL], By[MAXBARREL], Bdx[MAXBARREL];
var	Nb, Bmax, Bdelay, Bcad;

var	Dropx[MAXDROP], Dropy[MAXDROP], Ndrop, Dropno;

var	Left, Right, Up, Down, Jump;
var	Delay, Mdelay;

var	Px, Py, Pdx;
var	Lads, Score, Level, Timer, Timer2, Gold;

length(s) return t.memscan(s, 0, 32767);

init() do var i;
	ie (VT52) con.setup([	24,
				"\eH\eJ",
				"\eK",
				"\eY%yc\s%xc\s",
				"\eI",
				"",
				""]);
	else
		con.setup([	24,
				"\e[H\e[J",
				"\e[K",
				"\e[%yd1;%xd1H",
				"\eM",
				"\e[?25l",
				"\e[?25h"]);
	con.csroff();
	for (i=0, 24) Screen[i] := @Buffer::(i*80);
	Left := 'j';
	Right := 'l';
	Up := 'i';
	Down := 'k';
	Jump := '\s';
	Delay := 125;
	if (Mdelay < 0) Mdelay := 2000;
	con.calibrate(Mdelay);
	Jumpd := [ 0, %1, %1, 0, 0, 1, 1 ];
	Levels := [
		[ [ 'P',  5, 10     ],
		  [ '=',  7,  3, 67 ],
		  [ '=',  0,  7, 46 ],
		  [ '=', 49,  7, 80 ],
		  [ '=',  7, 11, 37 ],
		  [ '=', 39, 11, 62 ],
		  [ '=',  0, 15, 24 ],
		  [ '=', 25, 15, 47 ],
		  [ '=', 48, 15, 72 ],
		  [ 'H', 57,  0,  3 ],
		  [ 'H', 16,  2,  7 ],
		  [ 'H', 27,  6,  9 ],
		  [ 'H', 16, 10, 15 ],
		  [ 'H', 57,  6, 11 ],
		  [ 'H', 57, 14, 19 ],
		  [ 'V', 39,  0,  0 ],
		  [ '*',  0, 18,  0 ],
		  [ '*', 79, 18,  0 ],
		  [ '&', 16,  8,  0 ],
		  [ '$', 57,  0,  0 ],
		  [ '|', 68,  8,  9 ],
		  [ '|', 76,  8,  9 ],
		  [ 'g',  5, 18,  0 ],
		  [ 'T', 67,  9,  "Easy Street" ],
		  0 ],
		[ [ 'P',  5, 10     ],
		  [ '=',  0,  3, 26 ],
		  [ '=', 27,  3, 53 ],
		  [ '=', 54,  3, 79 ],
		  [ '=',  0,  7, 24 ],
		  [ '=', 25,  7, 30 ],
		  [ '=', 32,  7, 53 ],
		  [ '=', 54,  7, 79 ],
		  [ '=',  0, 11, 24 ],
		  [ '=', 27, 11, 32 ],
		  [ '=', 34, 11, 52 ],
		  [ '=', 35, 11, 52 ],
		  [ '=', 54, 11, 79 ],
		  [ '=',  0, 15, 23 ],
		  [ '=', 25, 15, 32 ],
		  [ '=', 36, 15, 51 ],
		  [ '=', 54, 15, 79 ],
		  [ '=',  0, 17, 14 ],
		  [ 'H',  5,  2,  7 ],
		  [ 'H',  5, 10, 15 ],
		  [ 'H', 76,  6, 11 ],
		  [ 'H', 76, 14, 19 ],
		  [ 'H', 76,  0,  3 ],
		  [ 'H', 76,  0,  3 ],
		  [ '|', 13,  2,  3 ],
		  [ '|', 70,  2,  3 ],
		  [ '|', 25,  6,  7 ],
		  [ '|', 31,  9, 11 ],
		  [ '|', 25, 13, 15 ],
		  [ '|', 33, 17, 19 ],
		  [ '|', 13, 18, 19 ],
		  [ 'V', 14,  0, 20 ],
		  [ 'V', 69,  0, 20 ],
		  [ '&', 70,  0, 20 ],
		  [ '&', 23,  6, 20 ],
		  [ '*', 25, 18, 20 ],
		  [ '*', 53, 18, 20 ],
		  [ '$', 76,  0,  0 ],
		  [ 'g', 18, 18, 20 ],
		  [ 'T',  1, 18, "Long Island" ],
		  0 ],
		[ [ 'P',  5, 10     ],
		  [ '=',  0,  3, 14 ],
		  [ '=',  5,  6, 19 ],
		  [ '=', 22,  6, 26 ],
		  [ '=', 30,  6, 33 ],
		  [ '=', 36,  6, 41 ],
		  [ '=', 44,  6, 47 ],
		  [ '=', 50,  6, 54 ],
		  [ '=', 58,  6, 69 ],
		  [ '=', 10, 10, 63 ],
		  [ '=', 64, 10, 75 ],
		  [ '=', 14, 14, 78 ],
		  [ '=', 62,  3, 79 ],
		  [ 'H', 10,  2,  6 ],
		  [ 'H', 63,  2,  6 ],
		  [ 'H', 78,  2,  7 ],
		  [ 'H', 32,  9, 14 ],
		  [ 'H', 50, 13, 17 ],
		  [ 'H', 70, 13, 19 ],
		  [ '|', 10,  9, 10 ],
		  [ '|', 14, 13, 14 ],
		  [ '|', 70,  8, 10 ],
		  [ '^', 19,  7, 22 ],
		  [ '^', 26,  7, 30 ],
		  [ '^', 33,  7, 36 ],
		  [ '^', 41,  7, 44 ],
		  [ '^', 47,  7, 50 ],
		  [ '^', 54,  7, 58 ],
		  [ '^', 31, 18, 34 ],
		  [ '^', 32, 17, 33 ],
		  [ '*',  0, 18,  0 ],
		  [ '*', 79, 18,  0 ],
		  [ 'V', 28,  0,  0 ],
		  [ 'V', 42,  0,  0 ],
		  [ 'V', 56,  0,  0 ],
		  [ '&', 40,  5,  0 ],
		  [ '&', 13, 11,  0 ],
		  [ '&', 62,  9,  0 ],
		  [ '&', 32, 16,  0 ],
		  [ '&', 74,  9,  0 ],
		  [ '$', 78,  7,  0 ],
		  [ 'g',  5,  2,  0 ],
		  [ 'U',  4,  7,  "Ghost Town" ],
		  0 ],
		[ [ 'P',  5, 10     ],
		  [ '=',  0,  3, 46 ],
		  [ '=', 51,  3, 74 ],
		  [ '=', 36,  5, 54 ],
		  [ '=', 17,  7, 53 ],
		  [ '=', 15,  9, 24 ],
		  [ '=', 39,  9, 52 ],
		  [ '=', 17, 11, 54 ],
		  [ '=', 15, 15, 59 ],
		  [ '=', 65, 11, 80 ],
		  [ '=', 68, 17, 79 ],
		  [ '=',  0, 14, 11 ],
		  [ '=',  4, 16,  7 ],
		  [ 'H',  5,  2, 12 ],
		  [ 'H', 19,  2,  7 ],
		  [ 'H', 62,  2, 19 ],
		  [ 'H', 51,  8, 11 ],
		  [ 'H', 40,  6,  9 ],
		  [ 'H', 17, 10, 15 ],
		  [ 'H', 74,  7, 11 ],
		  [ 'H', 71, 16, 19 ],
		  [ '|', 45,  2,  3 ],
		  [ '|', 36,  4,  5 ],
		  [ '|', 15,  8,  9 ],
		  [ '|', 53, 10, 11 ],
		  [ '|', 58, 13, 15 ],
		  [ '|',  0, 15, 19 ],
		  [ '|', 10, 15, 19 ],
		  [ '-', 11,  3, 13 ],
		  [ '-', 54,  3, 58 ],
		  [ '-', 24,  7, 27 ],
		  [ '-', 30,  7, 34 ],
		  [ '-', 24,  9, 27 ],
		  [ '-', 34,  9, 39 ],
		  [ '-', 27, 11, 31 ],
		  [ '-', 34, 11, 38 ],
		  [ '-', 31, 15, 34 ],
		  [ '-', 37, 15, 40 ],
		  [ '-',  4, 14,  7 ],
		  [ '&', 37,  4,  0 ],
		  [ '&', 38,  4,  0 ],
		  [ '&', 27,  9,  0 ],
		  [ '&', 58, 12,  0 ],
		  [ '&', 58, 12,  0 ],
		  [ 'V', 44,  0,  0 ],
		  [ 'V', 68,  0,  0 ],
		  [ '*',  1, 18,  0 ],
		  [ '*',  9, 18,  0 ],
		  [ '*', 14, 18,  0 ],
		  [ '*', 31, 18,  0 ],
		  [ '*', 39, 18,  0 ],
		  [ '*', 61, 18,  0 ],
		  [ '*', 70, 18,  0 ],
		  [ '$',  5, 18,  0 ],
		  [ 'g', 76, 16,  0 ],
		  [ 'T', 43,  6,  "tunnel" ],
		  [ 'T', 43,  8,  "vision" ],
		  0 ],
		[ [ 'P',  5, 10     ],
		  [ '=',  9,  3, 21 ],
		  [ '=', 27,  3, 41 ],
		  [ '=',  0,  7, 45 ],
		  [ '=', 66,  4, 79 ],
		  [ '=', 53,  9, 73 ],
		  [ '=', 41, 11, 44 ],
		  [ '=', 13, 12, 42 ],
		  [ '=', 55, 13, 79 ],
		  [ '=',  3, 16, 74 ],
		  [ 'H',  9,  0,  3 ],
		  [ 'H', 41,  2,  7 ],
		  [ 'H', 68,  3,  9 ],
		  [ 'H', 58,  8, 13 ],
		  [ 'H', 39, 11, 16 ],
		  [ 'H', 41, 10, 12 ],
		  [ 'H', 72, 12, 16 ],
		  [ 'H',  3,  6, 16 ],
		  [ '-', 18,  7, 29 ],
		  [ '-', 44, 11, 53 ],
		  [ '-', 46, 13, 55 ],
		  [ '-', 13, 16, 38 ],
		  [ '-', 45, 16, 55 ],
		  [ '^',  0, 18,  3 ],
		  [ '^', 13, 18, 38 ],
		  [ '^', 45, 18, 55 ],
		  [ '^', 75, 18, 80 ],
		  [ '^', 27, 11, 32 ],
		  [ '^', 13, 11, 18 ],
		  [ '.', 26, 11,  0 ],
		  [ '.', 32, 11,  0 ],
		  [ '.', 50,  6,  0 ],
		  [ '.', 26,  2,  0 ],
		  [ 'V', 61,  1,  0 ],
		  [ 'V', 29,  4,  0 ],
		  [ '*',  3, 18,  0 ],
		  [ '*', 38, 18,  0 ],
		  [ '*', 44, 18,  0 ],
		  [ '*', 55, 18,  0 ],
		  [ '*', 74, 18,  0 ],
		  [ '&',  9,  4,  0 ],
		  [ '&', 18, 11,  0 ],
		  [ '&', 46, 12,  0 ],
		  [ '&', 30, 15,  0 ],
		  [ '&', 52, 15,  0 ],
		  [ '$',  9,  0,  0 ],
		  [ 'g', 73,  3,  0 ],
		  [ 'T', 56, 18,  "Point of No Return" ],
		  0 ],
		[ [ 'P',  5, 10     ],
		  [ '=',  4,  3, 19 ],
		  [ '=', 19,  4, 24 ],
		  [ '=', 38,  3, 59 ],
		  [ '=', 63,  4, 80 ],
		  [ '=', 11,  8, 22 ],
		  [ '=', 30,  9, 52 ],
		  [ '=', 63,  9, 69 ],
		  [ '=', 67, 11, 77 ],
		  [ '=',  0, 12,  7 ],
		  [ '=', 23, 12, 45 ],
		  [ '=',  9, 15, 19 ],
		  [ '=', 22, 15, 29 ],
		  [ '=', 32, 16, 44 ],
		  [ '=', 51, 16, 56 ],
		  [ '=', 61, 16, 62 ],
		  [ '=', 67, 16, 80 ],
		  [ '=', 23,  3, 34 ],
		  [ 'H',  3,  2, 11 ],
		  [ 'H', 15,  7, 15 ],
		  [ 'H', 33, 15, 19 ],
		  [ 'H', 44, 11, 15 ],
		  [ 'H', 46,  7,  9 ],
		  [ 'H', 51,  8, 15 ],
		  [ 'H', 51,  8, 14 ],
		  [ 'H', 68,  3,  9 ],
		  [ 'H', 72, 10, 16 ],
		  [ 'H', 23,  2,  4 ],
		  [ '|', 18,  4,  5 ],
		  [ '|', 23, 10, 12 ],
		  [ '|', 44,  6,  9 ],
		  [ '|', 44,  6,  9 ],
		  [ 'V', 63,  0,  0 ],
		  [ 'V', 44,  4,  0 ],
		  [ '&', 45,  8,  0 ],
		  [ '&', 22, 10,  0 ],
		  [ '&', 22, 11,  0 ],
		  [ '&',  7, 12,  0 ],
		  [ '&', 51, 15,  0 ],
		  [ '&', 44, 15,  0 ],
		  [ '$', 46,  6,  0 ],
		  [ '*',  0, 18,  0 ],
		  [ '*', 79, 18,  0 ],
		  [ 'g', 35,  8,  0 ],
		  [ 'T', 46,  2,  ">mmmmmmmm" ],
		  [ 'T', 57, 11,  "}{" ],
		  [ 'T', 38, 15,  "<>" ],
		  [ 'T',  5, 17,  "}i{" ],
		  [ 'T', 34,  4,  "\\  /" ],
		  [ 'T', 35,  5,  "\\/" ],
		  [ 'T', 34,  0,  "oooo" ],
		  [ 'T', 33,  1,  "OO  OO" ],
		  [ 'T',  8,  0,  "Bug City" ],
		  0 ],
		[ [ 'P',  6, 10     ],
		  [ '=',  1, 16, 22 ],
		  [ '=',  3, 12, 24 ],
		  [ '=',  1,  8, 22 ],
		  [ '=',  3,  4, 22 ],
		  [ '=', 22,  3, 26 ],
		  [ '-', 10,  4, 15 ],
		  [ '-', 26,  5, 27 ],
		  [ '-', 26,  7, 27 ],
		  [ '-', 26,  9, 27 ],
		  [ '-', 26, 11, 27 ],
		  [ '-', 26, 13, 27 ],
		  [ '-', 26, 15, 27 ],
		  [ '|',  1, 15, 16 ],
		  [ '|', 23, 11, 12 ],
		  [ '|',  1,  7,  8 ],
		  [ '|', 21,  3,  4 ],
		  [ '|', 25,  4, 16 ],
		  [ '|', 27,  3, 16 ],
		  [ 'H', 20, 15, 19 ],
		  [ 'H',  4, 11, 16 ],
		  [ 'H', 20,  7, 12 ],
		  [ 'H',  4,  3,  8 ],
		  [ '^', 11, 11, 14 ],
		  [ '^', 27,  0, 28 ],
		  [ '^', 27,  1, 28 ],
		  [ '^', 27,  2, 28 ],
		  [ '&', 26,  4,  0 ],
		  [ '&', 26,  6,  0 ],
		  [ '&', 26,  8,  0 ],
		  [ '&', 26, 10,  0 ],
		  [ '&', 26, 12,  0 ],
		  [ '&', 26, 14,  0 ],
		  [ 'V', 20,  0,  0 ],
		  [ '*', 17, 18,  0 ],
		  [ '*', 23, 18,  0 ],
		  [ '=', 27, 16, 48 ],
		  [ '=', 29, 12, 50 ],
		  [ '=', 28,  8, 48 ],
		  [ '=', 29,  4, 48 ],
		  [ '=', 48,  3, 52 ],
		  [ '-', 36,  4, 41 ],
		  [ '-', 52,  5, 27 ],
		  [ '-', 52,  8, 27 ],
		  [ '-', 52, 11, 27 ],
		  [ '-', 52, 14, 27 ],
		  [ '^', 37,  7, 40 ],
		  [ '|', 27, 15, 16 ],
		  [ '|', 49, 11, 12 ],
		  [ '|', 27,  7,  8 ],
		  [ '|', 47,  3,  4 ],
		  [ '|', 51,  4, 19 ],
		  [ '|', 53,  3, 16 ],
		  [ 'H', 46, 15, 19 ],
		  [ 'H', 30, 11, 16 ],
		  [ 'H', 46,  7, 12 ],
		  [ 'H', 30,  3,  8 ],
		  [ '^', 53,  0, 54 ],
		  [ '^', 53,  1, 54 ],
		  [ '^', 53,  2, 54 ],
		  [ 'V', 46,  0,  0 ],
		  [ '*', 43, 18,  0 ],
		  [ '*', 49, 18,  0 ],
		  [ '=', 53, 16, 74 ],
		  [ '=', 55, 12, 76 ],
		  [ '=', 54,  8, 74 ],
		  [ '=', 55,  4, 74 ],
		  [ '=', 74,  3, 78 ],
		  [ '-', 62,  4, 67 ],
		  [ '-', 78,  4, 79 ],
		  [ '-', 78,  6, 79 ],
		  [ '-', 78,  8, 79 ],
		  [ '-', 78, 10, 79 ],
		  [ '-', 78, 12, 79 ],
		  [ '-', 78, 14, 79 ],
		  [ '-', 78, 16, 79 ],
		  [ '^', 63,  7, 66 ],
		  [ '^', 63, 15, 66 ],
		  [ '|', 53, 15, 16 ],
		  [ '|', 75, 11, 12 ],
		  [ '|', 53,  7,  8 ],
		  [ '|', 73,  3,  4 ],
		  [ '|', 77,  4, 19 ],
		  [ '|', 79,  3, 19 ],
		  [ 'H', 72, 15, 19 ],
		  [ 'H', 56, 11, 16 ],
		  [ 'H', 72,  7, 12 ],
		  [ 'H', 56,  3,  8 ],
		  [ 'V', 72,  0,  0 ],
		  [ '*', 69, 18,  0 ],
		  [ '*', 75, 18,  0 ],
		  [ '$', 78, 18,  0 ],
		  [ 'T', 34,  0,  "F R E E" ],
		  [ 'T', 34,  1,  "F A L L" ],
		  [ 'g', 30, 18,  0 ],
		  0 ],
		0
	];
end

fini() do
	con.csron();
	con.clrscr();
	con.shutdown();
end

var ntoa_buf::9;
ntoa(n, k) do var i;
	i := 9;
	ntoa_buf::i := 0;
	while (i = 9 \/ n) do
		i := i-1;
		ntoa_buf::i := n mod 10 + '0';
		n := n / 10;
		k := k-1;
	end
	while (k > 0) do
		i := i-1;
		ntoa_buf::i := '0';
		k := k-1;
	end
	return @ntoa_buf::i;
end

drain() while (con.pollkey()) ;

lfsr(k) do var s, x;
	s := [ 123 ];
	x := s[0] >> 1 ^ s[0] & 1 << 14;
	s[0] := s[0] >> 1 | x;
	return s[0] mod k;
end

putscr(x, y, c) Screen[y]::x := c;

getscr(x, y) return Screen[y]::x;

putc(x, y, c) do
	con.move(x, y);
	con.wrch(c);
	if (Register) putscr(x, y, c);
end

puts(x, y, s) do var i, k;
	k := length(s);
	for (i=0, k) putc(x+i, y, s::i);
end

putdown(x, y, s) do var i, k;
	k := length(s);
	for (i=0, k) putc(x, y+i, s::i);
end

hline(x, y, dx, c) do var i;
	for (i=x, dx) putc(i, y, c);
end

vline(x, y, dy, c) do var i;
	for (i=y, dy) putc(x, i, c);
end

setpxy(x, y) do
	Px := x;
	Py := y;
end

setdrop(x, y) do
	Dropx[Ndrop] := x;
	Dropy[Ndrop] := y;
	Ndrop := Ndrop + 1;
end

setparms(p) do
	Bmax := p[P_BMAX];
	Bcad := p[P_BCAD];
end

drawlevel(lev) do var i, p;
	con.clrscr();
	Ndrop := 0;
	Register := %1;
	t.memfill(Buffer, '\s', 24*80);
	i := 0;
	while (lev[i]) do
		p := lev[i];
		     ie (p[0] = '=') hline(p[1], p[2], p[3], '=');
		else ie (p[0] = '^') hline(p[1], p[2], p[3], '^');
		else ie (p[0] = '-') hline(p[1], p[2], p[3], '-');
		else ie (p[0] = 'H') vline(p[1], p[2], p[3], 'H');
		else ie (p[0] = '|') vline(p[1], p[2], p[3], '|');
		else ie (p[0] = 'T') puts(p[1], p[2], p[3]);
		else ie (p[0] = 'U') putdown(p[1], p[2], p[3]);
		else ie (p[0] = 'g') setpxy(p[1], p[2]);
		else ie (p[0] = 'P') setparms(p);
		else putc(p[1], p[2], p[0]);
		if (p[0] = 'V') setdrop(p[1], p[2]);
		i := i+1;
	end
	hline(0, 19, 80, '=');
	Register := 0;
end

message(s) do
	con.move(1, 22);
	con.writes(s);
end

clrmsg() do
	con.move(0, 22);
	con.clreol();
end

flash(s, n) do var i;
	for (i=0, n) do
		message(s);
		con.wait(Delay);
		clrmsg();
		con.wait(Delay);
	end
end

waitmsg(s) do var k;
	message(s);
	k := con.getkey();
	clrmsg();
	return k;
end

loadconf() do var fd;
	fd := t.open("ladders.cnf", T3X.OREAD);
	ie (fd < 0) do
		t.memfill(Scores, 0, 128);
		MDelay := %1;
	end
	else do
		t.read(fd, Scores, 128);
		t.close(fd);
		Mdelay := Scores::2 << 8 | Scores::1;
		con.calibrate(Mdelay);
	end
end

saveconf() do var fd;
	fd := t.open("ladders.cnf", T3X.OWRITE);
	if (fd < 0) do
		waitmsg("Could not save config -- press any key");
		return;
	end
	Scores::0 := VT52-> '5': '1';
	Scores::1 := Mdelay;
	Scores::2 := Mdelay >> 8;
	Scores::3 := Up;
	Scores::4 := Down;
	Scores::5 := Left;
	Scores::6 := Right;
	Scores::7 := Jump;
	t.write(fd, Scores, 128);
	t.close(fd);
end

clearbarrels() do var i;
	for (i=0, MAXBARREL) Bx[i] := %1;
end

newbarrel() do var i, dn;
	Bdelay := Bdelay + 1;
	if (Bdelay < Bcad) return;
	Bdelay := 0;
	if (Nb >= Bmax) return;
	for (i=0, Bmax)
		if (Bx[i] = %1) leave;
	Bx[i] := Dropx[Dropno];
	By[i] := Dropy[Dropno];
	if (lfsr(100) > 24) Dropno := Dropno + 1;
	if (Dropno >= Ndrop) Dropno := 0;
	Bdx[i] := lfsr(100) > 49-> %1: 1;
	Nb := Nb+1;
end

movebarrels() do var i, x, y, c;
	for (i=0, Bmax)
		if (Bx[i] >= 0) putc(Bx[i], By[i], getscr(Bx[i], By[i]));
	for (i=0, Bmax) do
		x := Bx[i];
		if (x >= 0) do
			y := By[i];
			c := getscr(x, y+1);
			ie (c = '\s' \/ c = '*') do
				y := y+1;
			end
			else ie (getscr(x, y) = '*') do
				x := %1;
				Nb := Nb-1;
			end
			else ie (getscr(x, y) = 'H') do
				ie (c = 'H' /\ getscr(x, y-1) = 'H') do
					y := y+1;
				end
				else ie (lfsr(100) > 84) do
					Bdx[i] := -Bdx[i];
					x := x + Bdx[i];
				end
				else ie (c = 'H' /\ lfsr(100) > 49) do
					y := y+1;
				end
				else do
					x := x + Bdx[i];
				end
			end
			else ie (getscr(x+Bdx[i], y) = '|') do
				Bdx[i] := -Bdx[i];
				x := x + Bdx[i];
			end
			else do
				x := x + Bdx[i];
				if (x < 1 \/ x > 78) Bdx[i] := -Bdx[i];
			end
			Bx[i] := x;
			By[i] := y;
		end
	end
	for (i=0, Bmax)
		if (Bx[i] >= 0) putc(Bx[i], By[i], 'o');
end

status() do
	con.move(1, 20);
	ie (Debug) do
		con.writes("Lads: INF\s\s\s\sLevel: ");
	end
	else do
		con.writes("Lads: ");
		con.writes(ntoa(Lads, 2));
		con.writes("\s\s\s\s\sLevel: ");
	end
	con.writes(ntoa(Level+1, 2));
	con.writes("\s\s\s\s\sScore: ");
	con.writes(ntoa(Score, 5));
	con.writes("\s\s\s\s\sGold: ");
	con.writes(ntoa(Gold, 4));
	con.writes("\s\s\s\s\sTime: ");
	con.writes(ntoa(Timer, 4));
end

forward() do var c;
	c := getscr(Px, Py+1);
	ie (t.memscan("\s*^&$", c, 5) \= -1) do
		Py := Py + 1;
	end
	else ie (c = '-') do
		putscr(Px, Py+1, '\s');
		putc(Px, Py+1, '\s');
		Px := Px + Pdx;
	end
	else do
		Px := Px + Pdx;
	end
end

grab() do
	putscr(Px, Py, '\s');
	Gold := Gold + 500;
	status();
end

canjump() do var c;
	if (getscr(Px+Pdx, Py-1) = '=') return 0;
	c := getscr(Px, Py+1);
	return c \= '\s' /\ c \= '^' -> 1 : 0;
end

die() do var i;
	for (i=0, 3) do
		putc(Px, Py, 'p');
		con.wait(Delay);
		putc(Px, Py, 'q');
		con.wait(Delay);
		putc(Px, Py, 'd');
		con.wait(Delay);
		putc(Px, Py, 'b');
		con.wait(Delay);
	end
	putc(Px, Py, 'x');
	if (\Debug) Lads := Lads - 1;
	status();
end

collision() do var i;
	for (i=0, Bmax) do
		if (By[i] \= Py) loop;
		if (Bx[i] = Px) do
			die();
			return %1;
		end
	end
end

pause() do var k;
	putc(Px, Py, 'g');
	k := waitmsg("Paused -- [y] = quit,  other = continue");
	putc(Px, Py, getscr(Px, Py));
	return k = 'y' \/ k = 'Y';
end

tick() do
	if (Timer) do
		Timer2 := Timer2-1;
		if (\Timer2) do
			Timer2 := 100;
			Timer := Timer-100;
			status();
		end
	end
end

playlevel(n) do var i, k, c, climbing;
	drawlevel(Levels[n mod MAXLEVEL]);
	for (i=MAXLEVEL, n+1, MAXLEVEL) do
		Bmax := Bmax + 2;
		Bcad := Bcad - 1;
	end
	drain();
	Pdx := 0;
	Jumpp := 0;
	Climbd := 0;
	Bdelay := 0;
	Nb := 0;
	Dropno := 0;
	Timer := 1500;
	Timer2 := 100;
	climbing := 0;
	clearbarrels();
	putc(Px, Py, 'g');
	status();
	for (i=0, 10) do
		message("Get Ready!");
		putc(Px, Py, '\s');
		con.wait(Delay);
		clrmsg();
		putc(Px, Py, 'g');
		con.wait(Delay);
	end
	k := con.getkey();
	while (1) do
		putc(Px, Py, Pdx<0-> 'q': Pdx>0-> 'p': 'g');
		newbarrel();
		if (collision()) return %1;
		movebarrels();
		if (collision()) return %1;
		con.move(Px, Py);
		con.wait(Delay);
		tick();
		if (\k) k := con.pollkey();
		if (k) lfsr(1);
		c := getscr(Px, Py);
		putc(Px, Py, getscr(Px, Py));
		     ie (k = Left /\ Jumpp < 3) Pdx := %1;
		else ie (k = Right /\ Jumpp < 3) Pdx := 1;
		else ie (k = Jump /\ \Jumpp) Jumpp := canjump();
		else ie (k = Up /\ \Jumpp) Climbd := %1;
		else ie (k = Down /\ \Jumpp) Climbd := 1;
		else ie (k = 'q') if (pause()) return 'Q';
		else if (k /\ \Jumpp) Pdx := 0;
		ie (Jumpp) do
			ie (	Jumpp > 4 /\
				getscr(Px+Pdx, Py+Jumpd[Jumpp]) = '=')
			do
				Jumpp := 0;
				forward();
			end
			else do
				Py := Py + Jumpd[Jumpp];
				Px := Px + Pdx;
				Jumpp := Jumpp+1;
				if (Jumpp = 7) Jumpp := 0;
				if (k = Up /\ getscr(Px, Py) = 'H') do
					Climbd := %1;
					Jumpp := 0;
				end
			end
		end
		else ie (Climbd) do
			c := getscr(Px, Py+Climbd);
			if (c = '$') leave;
			ie (c = 'H') do
				Py := Py + Climbd;
				if (\climbing) Pdx := 0;
				climbing := %1;
			end
			else do
				if (climbing) Climbd := 0;
				climbing := 0;
				forward();
			end
		end
		else do
			forward();
		end
		c := getscr(Px, Py);
		     ie (c = '|') Px := Px - Pdx;
		else ie (c = '&') grab();
		else ie (c = '$') leave;
		else ie (c = '.') Jumpp := 1;
		else if (c = '^') do die(); leave; end
		if (Px > 79) do
			Px := 79;
			Pdx := 0;
		end
		if (Px < 0) do
			Px := 0;
			Pdx := 0;
		end
		k := 0;
	end
	return c;
end

anykey() waitmsg("Press any key");

readcon(x, y, s, k) do var i, c;
	i := 0;
	while (1) do
		con.move(x+i, y);
		c := con.getkey();
		if (c = '\r' \/ c = '\n') return i;
		ie (i > 0 /\ (c = '\b' \/ c = 127)) do
			i := i-1;
			con.move(x+i, y);
			con.wrch('\s');
		end
		else if (i < k) do
			con.wrch(c);
			s::i := c;
			i := i+1;
		end
	end
end

enterscore(s) do var i, k;
	con.clrscr();
	con.csron();
	hline(8, 9, 39, '-');
	hline(8, 11, 39, '-');
	putc(8, 10, '|');
	putc(38, 10, '|');
	puts(10, 10, "Enter your name: ");
	k := readcon(27, 10, s, 10);
	con.csroff();
	if (k > 0 /\ s::(k-1) = '\n') s::(k-1) := '\s';
	for (i=k, 11) s::i := '\s';
	saveconf();
end

checkscore() do var i, n;
	if (Debug) return;
	for (i=16, 128, 16) do
		n := Scores::(i+1) << 8 | Scores::i;
		if (Score .> n) do
			t.memcopy(@Scores::(i+16), @Scores::i, 128-i-16);
			Scores::i := Score;
			Scores::(i+1) := Score >> 8;
			enterscore(@Scores::(i+4));
			return;
		end
	end
end

play(lev) do var k;
	Lads := 5;
	Score := 0;
	Gold := 0;
	Level := lev;
	while (Lads) do
		k := playlevel(Level);
		drain();
		if (k = 'Q') leave;
		if (k = '$') do
			Level := Level + 1;
			Timer := Timer / 100 * 100;
			while (Timer) do
				Score := Score + 100;
				Timer := Timer - 100;
				status();
				con.wait(Delay*2);
			end
			Score := Score + Gold;
		end
		Gold := 0;
		status();
		if (Lads) anykey();
	end
	if (\Lads) waitmsg("GAME OVER -- press any key");
	checkscore();
end

yesno() do var k;
	con.writes("? (y/n) ");
	while (1) do
		k := con.getkey();
		if (k = 'y' \/ k = 'Y') return %1;
		if (k = 'n' \/ k = 'N') return 0;
	end
end

calibrate() do var k, i, c, old;
	con.clrscr();
	con.csroff();
	c := '#';
	hline(15, 6, 65, '-');
	putc(25, 6, '|'); puts(25, 7, "1s");
	putc(35, 6, '|'); puts(35, 7, "2s");
	putc(45, 6, '|'); puts(45, 7, "3s");
	putc(55, 6, '|'); puts(55, 7, "4s");
	putc(65, 6, '|'); puts(65, 7, "5s");
	i := 0;
	old := Mdelay;
	puts(15,  3, "This should be about 5 seconds:");
	puts(15, 11, "[+]  = Slower (+100)    [-] = Faster (-100)");
	puts(15, 12, "[>]  = Slower  (+10)    [<] = Faster  (-10)");
	puts(15, 13, "[)]  = Slower   (+1)    [(] = Faster   (-1)");
	puts(15, 15, "[CR] = Save + exit      [q] = Quit (don't save)");
	puts(15, 17, "Is this too slow? Start LADDERS with /I");
	while (1) do
		if (i = 0) c := c = '#'-> ':': '#';
		puts(15, 9, "Delay: ");
		con.clreol();
		con.writes(ntoa(Mdelay, 0));
		if (Mdelay < 10)
			con.writes(" [Your machine is too slow!]");
		putc(15+i, 5, c);
		con.wrch(c);
		con.calibrate(Mdelay);
		con.wait(Delay);
		i := (i + 1) mod 50;
		k := con.pollkey();
		     ie (k = '+') Mdelay := Mdelay + 100;
		else ie (k = '-') Mdelay := Mdelay - 100;
		else ie (k = '>') Mdelay := Mdelay + 10;
		else ie (k = '<') Mdelay := Mdelay - 10;
		else ie (k = ')') Mdelay := Mdelay + 1;
		else ie (k = '(') Mdelay := Mdelay - 1;
		else ie (k = '\r' \/ k = '\n') leave;
		else if (k = 'q') do Mdelay := old; leave; end
		if (Mdelay < 0) Mdelay := 0;
	end
	con.csron();
	con.clrscr();
	con.calibrate(Mdelay);
end

getkey() do var k;
	while (1) do
		k := con.getkey();
		if (k < '\s' \/ k > '~') loop;
		ie (k = '\s')
			con.writes("Space");
		else
			con.wrch(k);
		return k;
	end
end

configure() do
	con.clrscr();
	con.csron();
	puts(2, 2, "Calibrate");
	if (yesno()) calibrate();
	puts(2,  4, "   \qUp\q key: "); Up := getkey();
	puts(2,  6, " \qDown\q key: "); Down := getkey();
	puts(2,  8, " \qLeft\q key: "); Left := getkey();
	puts(2, 10, "\qRight\q key: "); Right := getkey();
	puts(2, 12, " \qJump\q key: "); Jump := getkey();
	con.csroff();
	saveconf();
	anykey();
end

menu(x, y, v) do var i;
	i := 0;
	while (v[i]) do
		puts(x, i+y, v[i]);
		i := i+1;
	end
end

info() do var m;
	m := [
	"----------------",
	"| Instructions |",
	"----------------",
	"",
	"Move the player using the navigation keys, jump over gaps and",
	"obstacles, collect gold tokens, avoid barrels, and try to reach",
	"the exit.",
	"",
	"p q g  Player      (moving right, left, and resting)",
	"  =    Floor       (you walk on it)",
	"  H    Ladder      (can be climbed up and down)",
	"  |    Wall        (nothing goes through those)",
	"  -    Trapdoor    (you can walk on it only once)",
	"  ^    Death trap  (do not touch!)",
	"  .    Trampoline  (will make you jump)",
	"  o    Barrel      (avoid those)",
	"  V    Barrel drop (be careful around those!)",
	"  *    Barrel sink (barrels disappear here)",
	"  &    Gold token  (500 pts, collect them)",
	"  $    Exit        (door to the next level)",
	"",
	"Reaching the exit faster will increase your score.",
	0];
	con.clrscr();
	menu(8, 1, m);
	drain();
	con.getkey();
end

godmenu() do var gm, k;
	gm := [ "=======================",
		"||  G O D   M E N U  ||",
		"=======================",
		"",
		"1,2,3,4,5,6,7 - Go to level",
		"            D - Debug play, no scoring",
		"                (currently off)",
		"",
		"Other - exit", 0];
	con.clrscr();
	menu(20, 5, gm);
	while (1) do
		puts(47, 11, Debug-> "ON) ": "off)");
		con.move(32, 13);
		k := con.getkey();
		ie ('1' <= k /\ k <= '7') do
			play(k-'0'-1);
			leave;
		end
		else ie (k = 'D' \/ k = 'd') do
			Debug := \Debug;
		end
		else
			leave;
	end
end

showscores() do var i, j, n, b::11;
	j := 1;
	b::10 := 0;
	puts(40, 10, "THE MADDEST LADDERS");
	for (i=16, 128, 16) do
		puts(40, j+11, ntoa(j, 1));
		con.writes(". ");
		con.writes(ntoa(Scores::(i+1) << 8 | Scores::i, 5));
		con.writes(" ");
		t.memcopy(b, @Scores::(i+4), 10);
		con.writes(b);
		j := j+1;
	end
end

title() do var i, logo, about, k, red, god::4;
	logo := [
	"  /|  ,/ .--. ___",
	" / |.'/ /___//   \\",
	"/    / /   //___.'",
	"HH      =====  ======  ======  ======= ======   ======",
	"HH     HH   HH HH   HH HH   HH HH      HH   HH HH",
	"HH     ======= HH   HH HH   HH ======  ======   =====",
	"HH     HH   HH HH   HH HH   HH HH      HH  HH       HH",
	"====== HH   HH ======  ======  ======= HH   HH ======",
	"",
	"P - Play",
	"C - Configure",
	"I - Instructions",
	"E - Exit",
	"",
	"Controls:", 0];
	about := [
	"Inspired by the classic LADDER",
	"game by Yahoo Software, 1982.",
	0
	];
	red := 1;
	drain();
	while (1) do
		if (red) do
			con.clrscr();
			menu(13, 1, logo);
			menu(37, 20, about);
			puts(23, 15, "["); con.wrch(Up); con.wrch(']');
			putc(24, 16, '|');
			puts(18, 17, "["); con.wrch(Left); con.writes("]--[");
			con.wrch(Down); con.writes("]--[");
			con.wrch(Right); con.wrch(']');
			puts(15, 19, "["); con.wrch(Jump);
			con.writes("] = jump");
			puts(13, 20, "Other = stop");
			puts(15, 21, "[q] = pause/quit");
			showscores();
		end
		red := 1;
		k := con.getkey();
		god::0 := god::1;
		god::1 := god::2;
		god::2 := god::3;
		god::3 := k;
		if (	\t.memcomp(god, "god", 3) /\
			(god::3 = '\r' \/ god::3 = '\n'))
		do
			red := %1;
			godmenu();
			loop;
		end
		     ie (k = 'e') leave;
		else ie (k = 'c') configure();
		else ie (k = 'i') info();
		else ie (k = 'p') play(0);
		else red := 0;;
	end
end

do var a::3, ini, i, k;
	VT52 := 0;
	ini := 0;
	i := 1;
	loadconf();
	while (1) do
		k := t.getarg(i, a, 3);
		if (k < 1) leave;
		     ie (\t.memcomp(a, "/5", 3)) VT52 := %1;
		else if (\t.memcomp(a, "/I", 3)) ini := 10;
		i := i+1;
	end
	init();
	if (ini) do
		Mdelay := 10;
		con.calibrate(Mdelay);
	end
	title();
	fini();
end
